@@ -3,12 +3,12 @@
  * dynloader.c
  *	  This dynamic loader uses Andrew Yu's libdl-1.0 package for Ultrix 4.x.
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/port/dynloader/ultrix4.c,v 1.13 2001/02/10 02:31:26 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/port/dynloader/ultrix4.c,v 1.15 2002/06/20 20:29:33 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -55,10 +55,10 @@ pg_dlopen(char *filename)
 		/* list the undefined symbols, if any */
 		if (count)
 		{
-			elog(NOTICE, "dl: Undefined:");
+			elog(WARNING, "dl: Undefined:");
 			while (*list)
 			{
-				elog(NOTICE, "  %s", *list);
+				elog(WARNING, "  %s", *list);
 				list++;
 			}
 		}
