@@ -3,7 +3,7 @@
  *
  * Copyright 2000 by PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql/src/bin/psql/startup.c,v 1.54.2.1 2002/05/14 02:08:22 ishii Exp $
+ * $Header: /cvsroot/pgsql-server/src/bin/psql/startup.c,v 1.68 2002/10/18 22:05:36 petere Exp $
  */
 #include "postgres_fe.h"
 
@@ -21,9 +21,7 @@
 #include <getopt.h>
 #endif
 
-#ifdef ENABLE_NLS
 #include <locale.h>
-#endif
 
 #include "libpq-fe.h"
 
@@ -37,18 +35,14 @@
 #include "settings.h"
 #include "variables.h"
 
-#ifdef MULTIBYTE
 #include "mb/pg_wchar.h"
-#else
-/* XXX Grand unified hard-coded badness; this should go into libpq */
-#define pg_encoding_to_char(x) "SQL_ASCII"
-#endif
 
 /*
  * Global psql options
  */
 PsqlSettings pset;
 
+#define PSQLRC ".psqlrc"
 
 /*
  * Structures to pass information between the option parsing routine
@@ -105,8 +99,8 @@ main(int argc, char *argv[])
 	char	   *password = NULL;
 	bool		need_pass;
 
-#ifdef ENABLE_NLS
 	setlocale(LC_ALL, "");
+#ifdef ENABLE_NLS
 	bindtextdomain("psql", LOCALEDIR);
 	textdomain("psql");
 #endif
@@ -256,7 +250,7 @@ main(int argc, char *argv[])
 
 		if ((value = GetVariable(pset.vars, "ECHO")) && strcmp(value, "all") == 0)
 			puts(options.action_string);
-		successResult = HandleSlashCmds(options.action_string, NULL, NULL) != CMD_ERROR
+		successResult = HandleSlashCmds(options.action_string, NULL, NULL, NULL) != CMD_ERROR
 			? EXIT_SUCCESS : EXIT_FAILURE;
 	}
 
@@ -281,13 +275,13 @@ main(int argc, char *argv[])
 		pset.issuper = test_superuser(PQuser(pset.db));
 		if (!QUIET() && !pset.notty)
 		{
-			printf(gettext("Welcome to %s, the PostgreSQL interactive terminal.\n\n"
+			printf(gettext("Welcome to %s %s, the PostgreSQL interactive terminal.\n\n"
 						   "Type:  \\copyright for distribution terms\n"
 						   "       \\h for help with SQL commands\n"
 					   "       \\? for help on internal slash commands\n"
 			  "       \\g or terminate with semicolon to execute query\n"
 						   "       \\q to quit\n\n"),
-				   pset.progname);
+				   pset.progname, PG_VERSION);
 #ifdef USE_SSL
 			printSSLInfo();
 #endif
@@ -548,6 +542,11 @@ parse_psql_options(int argc, char *argv[], struct adhoc_opts * options)
 				}
 				break;
 #ifndef HAVE_GETOPT_LONG
+
+				/*
+				 * FreeBSD has a broken getopt that causes this test to
+				 * fail.
+				 */
 			case '-':
 				fprintf(stderr,
 						gettext("%s was compiled without support for long options.\n"
@@ -606,19 +605,20 @@ process_psqlrc(void)
 
 	if (home)
 	{
-		psqlrc = malloc(strlen(home) + 20);
+		psqlrc = malloc(strlen(home) + 1 + strlen(PSQLRC) + 1 +
+				 strlen(PG_VERSION) + 1);
 		if (!psqlrc)
 		{
 			fprintf(stderr, gettext("%s: out of memory\n"), pset.progname);
 			exit(EXIT_FAILURE);
 		}
 
-		sprintf(psqlrc, "%s/.psqlrc-" PG_VERSION, home);
+		sprintf(psqlrc, "%s/%s-%s", home, PSQLRC, PG_VERSION);
 		if (access(psqlrc, R_OK) == 0)
 			process_file(psqlrc);
 		else
 		{
-			sprintf(psqlrc, "%s/.psqlrc", home);
+			sprintf(psqlrc, "%s/%s", home, PSQLRC);
 			if (access(psqlrc, R_OK) == 0)
 				process_file(psqlrc);
 		}
@@ -637,41 +637,9 @@ showVersion(void)
 {
 	puts("psql (PostgreSQL) " PG_VERSION);
 
-#if defined(USE_READLINE) || defined (USE_HISTORY) || defined(MULTIBYTE)
-	fputs(gettext("contains support for: "), stdout);
-
-#ifdef USE_READLINE
-	fputs(gettext("readline"), stdout);
-#define _Feature
+#if defined(USE_READLINE)
+	puts(gettext("contains support for command-line editing"));
 #endif
-
-#ifdef USE_HISTORY
-#ifdef _Feature
-	fputs(", ", stdout);
-#else
-#define _Feature
-#endif
-	fputs(gettext("history"), stdout);
-#endif
-
-#ifdef MULTIBYTE
-#ifdef _Feature
-	fputs(", ", stdout);
-#else
-#define _Feature
-#endif
-	fputs(gettext("multibyte"), stdout);
-#endif
-
-#undef _Feature
-
-	puts("");
-#endif
-
-	puts(gettext("Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group\n"
-				 "Portions Copyright (c) 1996, Regents of the University of California\n"
-	"Read the file COPYRIGHT or use the command \\copyright to see the\n"
-				 "usage and distribution terms."));
 }
 
 
