@@ -4,21 +4,17 @@
  *
  *
  *
- * Copyright (c) 1994, Regents of the University of California
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: nodeHash.h,v 1.13 1999/05/25 16:13:55 momjian Exp $
+ * $Id: nodeHash.h,v 1.24 2002/06/20 20:29:49 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #ifndef NODEHASH_H
 #define NODEHASH_H
 
-#include "executor/hashjoin.h"
-#include "executor/tuptable.h"
-#include "nodes/execnodes.h"
-#include "nodes/pg_list.h"
 #include "nodes/plannodes.h"
-#include "utils/syscache.h"
 
 extern TupleTableSlot *ExecHash(Hash *node);
 extern bool ExecInitHash(Hash *node, EState *estate, Plan *parent);
@@ -26,13 +22,19 @@ extern int	ExecCountSlotsHash(Hash *node);
 extern void ExecEndHash(Hash *node);
 extern HashJoinTable ExecHashTableCreate(Hash *node);
 extern void ExecHashTableDestroy(HashJoinTable hashtable);
-extern void ExecHashTableInsert(HashJoinTable hashtable, ExprContext *econtext,
-					Var *hashkey);
-extern int ExecHashGetBucket(HashJoinTable hashtable, ExprContext *econtext,
-				  Var *hashkey);
+extern void ExecHashTableInsert(HashJoinTable hashtable,
+					ExprContext *econtext,
+					Node *hashkey);
+extern int ExecHashGetBucket(HashJoinTable hashtable,
+				  ExprContext *econtext,
+				  Node *hashkey);
 extern HeapTuple ExecScanHashBucket(HashJoinState *hjstate, List *hjclauses,
 				   ExprContext *econtext);
 extern void ExecHashTableReset(HashJoinTable hashtable, long ntuples);
 extern void ExecReScanHash(Hash *node, ExprContext *exprCtxt, Plan *parent);
+extern void ExecChooseHashTableSize(double ntuples, int tupwidth,
+						int *virtualbuckets,
+						int *physicalbuckets,
+						int *numbatches);
 
-#endif	 /* NODEHASH_H */
+#endif   /* NODEHASH_H */
