@@ -4,10 +4,10 @@
  *	  prototypes for clauses.c.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: clauses.h,v 1.50 2001/12/10 22:54:12 tgl Exp $
+ * $Id: clauses.h,v 1.54 2002/09/11 14:48:55 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -42,13 +42,14 @@ extern List *make_ands_implicit(Expr *clause);
 extern bool contain_agg_clause(Node *clause);
 extern List *pull_agg_clause(Node *clause);
 
-extern bool contain_iter_clause(Node *clause);
+extern bool expression_returns_set(Node *clause);
 
 extern bool contain_subplans(Node *clause);
 extern List *pull_subplans(Node *clause);
 extern void check_subplans_for_ungrouped_vars(Query *query);
 
-extern bool contain_noncachable_functions(Node *clause);
+extern bool contain_mutable_functions(Node *clause);
+extern bool contain_volatile_functions(Node *clause);
 
 extern bool is_pseudo_constant_clause(Node *clause);
 extern List *pull_constant_clauses(List *quals, List **constantQual);
@@ -65,10 +66,15 @@ extern bool expression_tree_walker(Node *node, bool (*walker) (),
 											   void *context);
 extern Node *expression_tree_mutator(Node *node, Node *(*mutator) (),
 												 void *context);
+
+/* flags bits for query_tree_walker and query_tree_mutator */
+#define QTW_IGNORE_SUBQUERIES	0x01	/* subqueries in rtable */
+#define QTW_IGNORE_JOINALIASES	0x02	/* JOIN alias var lists */
+
 extern bool query_tree_walker(Query *query, bool (*walker) (),
-									 void *context, bool visitQueryRTEs);
+							  void *context, int flags);
 extern void query_tree_mutator(Query *query, Node *(*mutator) (),
-									 void *context, bool visitQueryRTEs);
+							   void *context, int flags);
 
 #define is_subplan(clause)	((clause) != NULL && \
 							 IsA(clause, Expr) && \
