@@ -3,7 +3,7 @@
  *			  procedural language
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/pl/plpgsql/src/plpgsql.h,v 1.24 2001/11/29 22:57:37 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/pl/plpgsql/src/plpgsql.h,v 1.28 2002/09/12 00:24:09 momjian Exp $
  *
  *	  This software is copyrighted by Jan Wieck - Hamburg.
  *
@@ -40,8 +40,10 @@
 #include "postgres.h"
 
 #include "fmgr.h"
+#include "miscadmin.h"
 #include "executor/spi.h"
 #include "commands/trigger.h"
+#include "utils/tuplestore.h"
 
 /**********************************************************************
  * Definitions
@@ -90,6 +92,7 @@ enum
 	PLPGSQL_STMT_SELECT,
 	PLPGSQL_STMT_EXIT,
 	PLPGSQL_STMT_RETURN,
+	PLPGSQL_STMT_RETURN_NEXT,
 	PLPGSQL_STMT_RAISE,
 	PLPGSQL_STMT_EXECSQL,
 	PLPGSQL_STMT_DYNEXECUTE,
@@ -420,11 +423,18 @@ typedef struct
 {								/* RETURN statement			*/
 	int			cmd_type;
 	int			lineno;
-	bool		retistuple;
 	PLpgSQL_expr *expr;
 	int			retrecno;
 }	PLpgSQL_stmt_return;
 
+typedef struct
+{								/* RETURN NEXT statement */
+	int			cmd_type;
+	int			lineno;
+	PLpgSQL_rec *rec;
+	PLpgSQL_row *row;
+	PLpgSQL_expr *expr;
+}	PLpgSQL_stmt_return_next;
 
 typedef struct
 {								/* RAISE statement			*/
@@ -494,12 +504,19 @@ typedef struct
 {								/* Runtime execution data	*/
 	Datum		retval;
 	bool		retisnull;
-	Oid			rettype;
+	Oid			rettype;		/* type of current retval */
+
+	Oid			fn_rettype;		/* info about declared function rettype */
 	bool		retistuple;
-	TupleDesc	rettupdesc;
 	bool		retisset;
+
+	TupleDesc	rettupdesc;
 	char	   *exitlabel;
 
+	Tuplestorestate *tuple_store;		/* SRFs accumulate results here */
+	MemoryContext tuple_store_cxt;
+	ReturnSetInfo *rsi;
+
 	int			trig_nargs;
 	Datum	   *trig_argv;
 
@@ -547,11 +564,13 @@ extern PLpgSQL_function *plpgsql_curr_compile;
  */
 extern PLpgSQL_function *plpgsql_compile(Oid fn_oid, int functype);
 extern int	plpgsql_parse_word(char *word);
-extern int	plpgsql_parse_dblword(char *string);
-extern int	plpgsql_parse_tripword(char *string);
-extern int	plpgsql_parse_wordtype(char *string);
-extern int	plpgsql_parse_dblwordtype(char *string);
-extern int	plpgsql_parse_wordrowtype(char *string);
+extern int	plpgsql_parse_dblword(char *word);
+extern int	plpgsql_parse_tripword(char *word);
+extern int	plpgsql_parse_wordtype(char *word);
+extern int	plpgsql_parse_dblwordtype(char *word);
+extern int	plpgsql_parse_tripwordtype(char *word);
+extern int	plpgsql_parse_wordrowtype(char *word);
+extern int	plpgsql_parse_dblwordrowtype(char *word);
 extern PLpgSQL_type *plpgsql_parse_datatype(char *string);
 extern void plpgsql_adddatum(PLpgSQL_datum * new);
 extern int	plpgsql_add_initdatums(int **varnos);
@@ -598,7 +617,7 @@ extern void plpgsql_ns_rename(char *oldname, char *newname);
  * Other functions in pl_funcs.c
  * ----------
  */
-extern char *plpgsql_tolower(char *s);
+extern void plpgsql_convert_ident(const char *s, char **output, int numidents);
 extern const char *plpgsql_stmt_typename(PLpgSQL_stmt * stmt);
 extern void plpgsql_dumptree(PLpgSQL_function * func);
 
